/*
 *
 * Compile:

gcc -fPIC -c openfail.c -o openfail.o
gcc -shared -o openfail.so openfail.o

 * Environment:

    - OPENFAIL: the path prefix you want all open()s to fail for.
    - OPENFAIL_NOEXEC: 0/1, don't pass failures to child processes
    - OPENFAIL_VERBOSE: 0/1, report opens as they happen
 
 * Example:

    OPENFAIL=/dev/nst LD_PRELOAD=./openfail.so cat /dev/nst0

#
# LD_PRELOAD library for capturing "open"
#
# $Id:$
#
# Copyright (C) 2001-2006 Kees Cook
# kees@outflux.net, http://outflux.net/
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# http://www.gnu.org/copyleft/gpl.html

Apologies to Yggdrasil for using all their code. 
This is based on "logwrites"; also under the GNU GPL.

*/



#include <features.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


#ifdef DYNAMIC
# define __USE_GNU
# include <dlfcn.h>
#else
# include <syscall.h>
#endif


int active=0;
int initialized=0;

void initialize(void) {
	initialized=1;

	if (getenv("OPENFAIL")) {
		active=1;
        if (getenv("OPENFAIL_NOEXEC")) {
		    unsetenv("OPENFAIL");	// stop ourselves from going over exec
        }
	}
}

#ifdef DYNAMIC

static int (*old_open)(const char *path, int flags, int mode);
static int (*old___open)(const char *path, int flags, int mode);


int
_init(void) {
        old_open        = dlsym(RTLD_NEXT, "open");
        old___open      = dlsym(RTLD_NEXT, "__open");

	initialize();

	return 0;
}

#else

static int old_open(const char *path, int flags, int mode) {
        return syscall(SYS_open, path, flags, mode);
}
#define old___open old_open

#endif



int
myopen ( const char *filename, int flags, int mode ) {
    int length;
    int result;

    if (active) {
        if (getenv("OPENFAIL_VERBOSE")) {
            printf("Got: '%s'\n",filename);
        }

        length=strlen(getenv("OPENFAIL"));
        if (strncmp(filename,getenv("OPENFAIL"),length)==0) {
            errno=ENOENT;
            return -1;
        }
    }

    result = old_open(filename, flags, mode);
    return result;
}

int
open ( const char *filename, int flags, int mode ) {
    if (!initialized) initialize();

    return myopen(filename, flags, mode );
}

int
open64 ( const char *filename, int flags, int mode ) {
    if (!initialized) initialize();

    return myopen(filename, flags, mode );
}

#if defined(__GLIBC__) && (((__GLIBC__ == 2) && (__GLIBC_MINOR__ >= 1)) || __GLIBC__ > 2)
int
__open ( const char *filename, int flags, int mode ) {
    if (!initialized) initialize();

    return myopen(filename, flags, mode );
}

int
__open64 ( const char *filename, int flags, int mode ) {
    if (!initialized) initialize();

    return myopen(filename, flags, mode );
}
#endif
